/*This demo receives a comma separated string over the serial port
from the Grasshopper plugin for Rhino, which ultimately controls a 
pan/tilt servo.

Feel free to email andy@liftarchitects.com if you have questions about this code.
 */

#include <Servo.h>

#include <Wire.h>

 

#define BAUDRATE 19200

#define BUFFSIZE  12    // buffer one command at a time, 12 bytes is longer than the max length

char buffer[BUFFSIZE];  // this is the double buffer

uint8_t bufferidx = 0;

uint16_t Servo1Degrees, Servo2Degrees;

 

Servo myPanservo;  // create variable object to control the Pan servo

Servo myTiltservo;  // create variable object to control the Tilt servo

 

char *parseptr;

char buffidx;

 

void setup()          

{

  myPanservo.attach(9);  // attaches the servo on pin 9 to the servo object

  myTiltservo.attach(10);  // attaches the servo on pin 10 to the servo object

  Serial.begin(BAUDRATE);

 

}

 

void loop()                    

{

  char c;    // holds one character from the serial port

  if (Serial.available()) {

    c = Serial.read();      // read one character

    buffer[bufferidx] = c;  // add to buffer

    Serial.print(c);//write raw data to serial for testing

   

    if (c == '\n') {   

      buffer[bufferidx+1] = 0; // terminate it

      Serial.println(" eol");    // debug marker

 

      parseptr = buffer;    // offload the buffer into temp variable

 

      Servo1Degrees = parsedecimal(parseptr);    // parse out the first number, and offload to variable

 

      parseptr = strchr(parseptr, ',')+1;  // move past the ","

 

      Servo2Degrees = parsedecimal(parseptr);    // parse the second number

 

      myPanservo.write(Servo1Degrees); // tell Pan servo to go to position in variable 'Servo1Degrees'

      myTiltservo.write(Servo2Degrees); // tell Tilt servo to go to position in variable 'Servo2Degrees'

 

     

      bufferidx = 0;      // reset the buffer for the next read

      return;        //return so that we don't trigger the index increment below

    }

  

       

        // didn't get newline, need to read more from the buffer

    bufferidx++;    // increment the index for the next character

    if (bufferidx == BUFFSIZE-1) {  //if we get to the end of the buffer reset for safety

      Serial.print('!', BYTE);

      bufferidx = 0;

    }

  }

}

 

uint32_t parsedecimal(char *str) 

{

  uint32_t d = 0;

 

  while (str[0] != 0) {

    if ((str[0] > '9') || (str[0] < '0'))

      return d;

    d *= 10;

    d += str[0] - '0';

    str++;

  }

  return d;

}

 
